<?php
/*
 * system_be.inc
 *
 * part of pfSense plus (https://www.netgate.com)
 * Copyright (c) 2022-2023 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 */

require_once('pkg-utils.inc');
require_once('util.inc');

/* constants used to reference useful ZFS user properties */
DEFINE("ZFS_PROP_DESCR", "pfsense:description");
DEFINE("ZFS_PROP_VERSI", "pfsense:version");
DEFINE("ZFS_PROP_BOOT", "pfsense:lastbooted");

/* PHP8 polyfill */
if (!function_exists('str_starts_with')) {
	function str_starts_with($haystack, $needle) {
		return ((string)$needle !== '' && strncmp($haystack, $needle, strlen($needle)) === 0);
	}
}

/* 
 * Utility function to get a pkg version from a specific chroot path
 * Return the version string or false on failure
 */

function _be_get_pkg_version($chroot, $pkg_name)
{
	exec('/usr/local/sbin/pkg-static -c '
		.escapeshellarg($chroot).' '
		.'query %v '
		.escapeshellarg($pkg_name), $output, $rc);
	return ($rc === 0) ? trim($output[0]) : false;
}

/* Utility function to get the full dataset name for a given boot environment */
function _be_get_be_dataset($be_name)
{
	$list = [];

	if (!($be = libbe_init())) {
		goto bailout;
	}

	$list = be_get_bootenv_props($be);
bailout:
	return $list[$be_name]['dataset'] ?? false;
}

/*
 * Utility function to get a zfs property from a specific boot environment
 * Returns property value or null on failure
 */
function _be_zfs_get_property($be_name, $property)
{
	$rc = -1;

	if (!($target = _be_get_be_dataset($be_name))) {
		goto bailout;
	}

	exec('/sbin/zfs get -H -o value '
		.escapeshellarg($property).' '
		.escapeshellarg($target), $output, $rc);
bailout:
	return ($rc === 0) ? trim($output[0]) : null;
}

/* 
 * Utility function to set a zfs property on a specific boot environment
 * Returns true on success, false on failure
 */
function _be_zfs_set_property($be_name, $property, $value)
{
	$rc = -1;

	if (!($target = _be_get_be_dataset($be_name))) {
		goto bailout;
	}

	exec('/sbin/zfs set '
		.escapeshellarg($property).'='.escapeshellarg($value).' '
		.escapeshellarg($target), $output, $rc);
bailout:
	return (($rc === 0) && (_be_zfs_get_property($be_name, $property) === $value));
}

/*
 * GUI action for activating a boot environment
 * Returns an array of errors for presenting to the user
 */
function be_bootenv_do_activate($be_name, $temporary = false)
{
	$errors = [];

	/* initialize libbe */
	if (!($be = libbe_init())) {
		$errors[] = gettext('Unable to initialize libbe.');
		goto bailout;
	}

	/* activate boot environment */
	if (be_activate($be, $be_name, $temporary) !== BE_ERR_SUCCESS) {
		$errors[] = sprintf(gettext('Unable to activate boot environment "%s". Error: %d'),
				$be_name, libbe_errno($be));
		goto bailout;
	}
bailout:
	return $errors;
}

/*
 * GUI action for deleting a boot environment
 * Returns an array of errors for presenting to the user
 */
function be_bootenv_do_delete($be_name)
{
	$errors = [];

	/* initialize libbe */
	if (!($be = libbe_init())) {
		$errors[] = gettext('Unable to initialize libbe.');
		goto bailout;
	}

	/* handle case where we are trying to delete the next boot environment */
	if ((be_nextboot_name($be) === $be_name)
		&& (be_activate($be, be_active_name($be)) !== BE_ERR_SUCCESS)) {
		$errors[] = sprintf(gettext('Unable to activate boot environment "%s". Error %d'),
				be_active_name($be), libbe_errno($be));
		goto bailout;
	}

	/* refresh libbe internal state and destroy the boot environment */
	if (($be = libbe_init())
	    && be_destroy($be, $be_name, BE_DESTROY_ORIGIN) !== BE_ERR_SUCCESS) {
		$errors[] = sprintf(gettext('Unable to delete boot environment "%s". Error %d'),
				$be_name, libbe_errno($be));
		goto bailout;
	}
bailout:
	return $errors;
}

/* Utility function to sync zfs user properties */
function _be_sync_zfs_properties($be_name)
{
	/* initialize libbe */
	if (!($be = libbe_init())) {
		return false;
	}

	/* bailout if the boot environment doesn't exist */
	if (be_exists($be, $be_name) !== BE_ERR_SUCCESS) {
		return false;
	}

	/* get the list of boot environment and their properties */
	$list = be_get_bootenv_props($be);

	/* check if the boot environment is already mounted */
	$mount_path = $list[$be_name]['mounted'];
	if (!isset($list[$be_name]['mounted'])) {
		/* doesn't look like it, so we mount it here */
		$mount_path = be_mount($be, $be_name, "/tmp/{$be_name}", BE_MNT_DEEP);
	}

	/* nothing to chroot into, so just bailout... */
	if (!file_exists($mount_path)) {
		return false;
	}

	/* attempt to set pfsense:version */
	if ($ver = _be_get_pkg_version($mount_path, get_base_pkg_name())) {
		_be_zfs_set_property($be_name, ZFS_PROP_VERSI, $ver);
	}

	/* unmounting root is bad, so don't... */
	if ($mount_path !== '/') {
		be_unmount($be, $be_name, BE_MNT_FORCE);
	}

	/* looks good... */
	return true;
}

/* GUI action for creating a quick boot environment */
function be_bootenv_do_quick()
{
	$errors = [];

	/* initialize libbe */
	if (!($be = libbe_init())) {
		$errors[] = gettext('Unable to initialize libbe.');
		goto bailout;
	}

	/* the format string used to generate the boot environment name */
	$be_name = sprintf('quick-%s', date("YmdHis"));

	/* check if the generated name already exists for some reason */
	if (be_exists($be, $be_name) === BE_ERR_SUCCESS) {
		$errors[] = sprintf(gettext('Boot environment "%s" already exists.'), $be_name);
		goto bailout;
	}

	/* attempt to create the new boot environment */
	if (be_create($be, $be_name) !== BE_ERR_SUCCESS) {
		$errors[] = sprintf(gettext('Unable to create boot environment "%s". Error %d'),
				$be_name, libbe_errno($be));
		goto bailout;
	}

	/* if this isn't performed, the version property will be empty */
	_be_sync_zfs_properties($be_name);
bailout:
	return $errors;
}

/* Utility function to determine if the root dataset holds the given path */
function be_is_path_on_root($path)
{
	exec('/sbin/zfs list -H -t filesystem -o name -r '
		.escapeshellarg($path).' 2>/dev/null', $output, $rc);

	if (($rc === 0)
	    && $be = libbe_init()) {
		foreach ($output as $row) {
			if (str_starts_with($row, be_active_path($be))) {
				return true;
			}
		}
	}
	return false;
}

/* Utility function for checking BE/ZFS system sanity, (e.g. returns false on UFS) */
function be_bootenv_check()
{
	if (function_exists('libbe_check') && is_module_loaded('zfs')) {
		return	libbe_check()
				    && be_is_path_on_root('/cf')
				    && be_is_path_on_root('/var/db/pkg');
	}
	return false;
}

/* Utility function for determining the current/active BE */
function be_bootenv_get_current()
{
	return ($be = libbe_init()) ? be_active_name($be) : gettext('Unknown');
}

/* Utility function for determining the next BE */
function be_bootenv_get_next()
{
	return ($be = libbe_init()) ? be_nextboot_name($be) : gettext('Unknown');
}

/* GUI function for generating formatted list of BEs */
function be_gen_select_list()
{
	$list = [];
	if ($be = libbe_init()) {
		$bootenvs = be_get_bootenv_props($be);
		foreach ($bootenvs as $be_name => $props) {
			$active = $props['nextboot'] ? ' ('.gettext('Active').')' : null;
			$list[$be_name] = "{$be_name}{$active}";
		}
	}
	return $list;
}

/* Converts unix timestamps into formatted datetime strings */
function be_nicetime($timestamp, $format = 'Y-m-d H:i')
{
	if (!is_numeric($timestamp)) {
		return '-';
	}

	$date = date_create();
	date_timestamp_set($date, $timestamp);
	return date_format($date, $format);
}

function be_bootenv_get_config($be_name)
{
	/* short-circuit if the BE is null */
	if (is_null($be_name)) {
		goto bailout;
	}

	/* initialize libbe */
	if (!($be = libbe_init())) {
		goto bailout;
	}

	/* check if the boot environment exists */
	if (be_exists($be, $be_name) !== BE_ERR_SUCCESS) {
		goto bailout;
	}

	/* grab the list of bootenv properties */
	$list = be_bootenv_list();

	/* return the list of config properties */
	return [
		'descr'		=> $list[$be_name]['descr'],
		'old_name'	=> $be_name,
		'name'		=> $be_name
	];
bailout:
	return false;
}

function be_bootenv_do_save($post)
{
	/* initialize libbe */
	if (!($be = libbe_init())) {
		$errors[] = gettext('Unable to initialize libbe.');
		goto bailout;
	}

	/* if this is just a re-save, validate it and move on */
	if (isset($post['old_name'])
	    && ($post['old_name'] === $post['name'])
	    && be_exists($be, $post['name']) === BE_ERR_SUCCESS) {
		goto sync;
	}

	/* if the name has changed, we validate it and rename it */
	if (isset($post['old_name'])
	    && ($post['old_name'] !== $post['name'])
	    && be_exists($be, $post['old_name']) === BE_ERR_SUCCESS) {
		if (be_rename($be, $post['old_name'], $post['name']) !== BE_ERR_SUCCESS) {
			$errors[] = sprintf(gettext('Unable to rename boot environment "%s". Error %d'),
					$post['old_name'], libbe_errno($be));
			$post['name'] = $post['old_name'];
			goto bailout;
		}
		goto sync;
	}

	/* check if the desired boot environment name aleady exists */
	if (be_exists($be, $post['name']) === BE_ERR_SUCCESS) {
		$errors[] = sprintf(gettext('Boot environment "%s" already exists.'), $post['name']);
		$post['name'] = '';
		goto bailout;
	}

	/* we need to create a new boot environment */
	if (be_create_from_existing($be, $post['name'], $post['from']) !== BE_ERR_SUCCESS) {
		$errors[] = sprintf(gettext('Unable to create boot environment "%s". Error %d'),
				$post['name'], libbe_errno($be));
		goto bailout;
	}
sync:
	/* we need to write the description property to the correct dataset */
	if (!_be_zfs_set_property($post['name'], ZFS_PROP_DESCR, $post['descr'])) {
		$errors[] = sprintf(gettext('Unable to set boot environment description "%s"'), $post['descr']);
		goto bailout;
	}

	/* if this isn't performed, the version property will be empty */
	_be_sync_zfs_properties($post['name']);
bailout:
	/* pass-through $post for updating the UI when we have errors */
	return [
		'input_errors'	=> $errors,
		'pconfig'	=> $post
	];
}

/*
 * Utility function to grab ZFS properties
 * This accepts an array of properties e.g. ['pfsense:version', pfsense:description']
 */
function _zfs_get_properties($props)
{
	$list = [];

	/* short-circuit if invalid props */
	if (!is_array($props)) {
		return false;
	}

	/* initialize libbe */
	if (!($be = libbe_init())) {
		return false;
	}

	/*
	 * This is a nasty hack because zfs(8) drops tab delimiters if the
	 * last column is empty. This is something to fix upstream later.
	 * $props becomes ['name', $props..., 'name'];
	 */
	array_unshift($props, 'name');
	array_push($props, 'name');

	exec('/sbin/zfs list -Hr -o '
		.escapeshellarg(implode(',', $props)).' '
		.be_root_path($be), $output, $rc);
	if ($rc <> 0 ) {
		return false;
	}

	/* build an associative array with 'name' property as the key */
	foreach ($output as $row) {
		$prop_values = explode("\t", $row);
		$list[$prop_values[0]] = array_combine($props, $prop_values);
	}

	/* looks good */
	return $list;
}

/* Utility function for generating the appropriate first-column status icon */
function _be_status_icon($be)
{
	$icon_format = '<i class="fa %s %s"></i>';

	if ($be['active']) {
		return sprintf($icon_format, 'fa-check-circle', 'text-success');
	}

	if ($be['nextboot']) {
		return sprintf($icon_format, 'fa-check-circle-o', 'text-success');
	}
}

/* Utility function for generating formatted list of BEs and properties */
function be_bootenv_list($sortbycol = 'creation')
{
	$list = [];

	/* initialize libbe */
	if (!($be = libbe_init())) {
		return $list;
	}

	/* grab the bootenv props from libbe */
	$list = be_get_bootenv_props($be);

	/* now grab the additional properties */
	$zfs_props = _zfs_get_properties([ZFS_PROP_DESCR, ZFS_PROP_VERSI, ZFS_PROP_BOOT]);

	/* walk the list and add some more useful bits */
	array_walk($list, function(&$x, $key) use ($zfs_props) {
		$x['activate_icon']	= $x['nextboot']
					? 'fa-star'
					: 'fa-star-o';
		$x['activate_title']	= $x['nextboot']
					? gettext('Next Boot Environment')
					: gettext('Activate Boot Environment');
		$x['lastbooted_nice']	= be_nicetime($zfs_props[$x['dataset']][ZFS_PROP_BOOT]);
		$x['creation_nice']	= be_nicetime($x['creation']);
		$x['descr']		= $zfs_props[$x['dataset']][ZFS_PROP_DESCR];
		$x['status_icon']	= _be_status_icon($x);
		$x['used_nice']		= be_nicenum($x['used']);
		$x['version']		= $zfs_props[$x['dataset']][ZFS_PROP_VERSI];
	});

	/* libbe assumes consumer will sort the resulting nvlist, so we sort it here */
	array_multisort(array_column($list, $sortbycol), SORT_ASC, $list);

	/* looks good */
	return $list;
}
